@@ -51,7 +51,7 @@ impl str::FromStr for SecretKey {
     fn from_str(s: &str) -> Result<SecretKey, Error> {
         let mut res = [0; constants::SECRET_KEY_SIZE];
         match from_hex(s, &mut res) {
-            Ok(constants::SECRET_KEY_SIZE) => Ok(SecretKey(res)),
+            Ok(constants::SECRET_KEY_SIZE) => SecretKey::from_slice(&res),
             _ => Err(Error::InvalidSecretKey)
         }
     }
@@ -65,6 +65,7 @@ pub const ONE_KEY: SecretKey = SecretKey([0, 0, 0, 0, 0, 0, 0, 0,
 
 /// A Secp256k1 public key, used for verification of signatures
 #[derive(Copy, Clone, PartialEq, Eq, Debug, Hash)]
+#[repr(transparent)]
 pub struct PublicKey(ffi::PublicKey);
 
 impl fmt::LowerHex for PublicKey {
@@ -389,14 +390,26 @@ impl PublicKey {
     /// the result would be the point at infinity, i.e. we are adding this point
     /// to its own negation
     pub fn combine(&self, other: &PublicKey) -> Result<PublicKey, Error> {
+        PublicKey::combine_keys(&[self, other])
+    }
+
+    /// Adds the keys in the provided slice together, returning the sum. Returns
+    /// an error if the result would be the point at infinity, i.e. we are adding
+    /// a point to its own negation
+    pub fn combine_keys(keys: &[&PublicKey]) -> Result<PublicKey, Error> {
+        use core::mem::transmute;
+        use core::i32::MAX;
+
+        debug_assert!(keys.len() < MAX as usize);
         unsafe {
             let mut ret = ffi::PublicKey::new();
-            let ptrs = [self.as_c_ptr(), other.as_c_ptr()];
+            let ptrs : &[*const ffi::PublicKey] =
+                transmute::<&[&PublicKey], &[*const ffi::PublicKey]>(keys);
             if ffi::secp256k1_ec_pubkey_combine(
                 ffi::secp256k1_context_no_precomp,
                 &mut ret,
                 ptrs.as_c_ptr(),
-                2
+                keys.len() as i32
             ) == 1
             {
                 Ok(PublicKey(ret))
@@ -525,6 +538,10 @@ mod test {
     fn invalid_secret_key() {
         // Zero
         assert_eq!(SecretKey::from_slice(&[0; 32]), Err(InvalidSecretKey));
+        assert_eq!(
+            SecretKey::from_str(&format!("0000000000000000000000000000000000000000000000000000000000000000")),
+            Err(InvalidSecretKey)
+        );
         // -1
         assert_eq!(SecretKey::from_slice(&[0xff; 32]), Err(InvalidSecretKey));
         // Top of range
@@ -673,7 +690,13 @@ mod test {
 
         let s = Secp256k1::signing_only();
         let sk = SecretKey::from_slice(&SK_BYTES).expect("sk");
+
+        // In fuzzing mode secret->public key derivation is different, so
+        // hard-code the epected result.
+        #[cfg(not(fuzzing))]
         let pk = PublicKey::from_secret_key(&s, &sk);
+        #[cfg(fuzzing)]
+        let pk = PublicKey::from_slice(&[0x02, 0x18, 0x84, 0x57, 0x81, 0xf6, 0x31, 0xc4, 0x8f, 0x1c, 0x97, 0x09, 0xe2, 0x30, 0x92, 0x06, 0x7d, 0x06, 0x83, 0x7f, 0x30, 0xaa, 0x0c, 0xd0, 0x54, 0x4a, 0xc8, 0x87, 0xfe, 0x91, 0xdd, 0xd1, 0x66]).expect("pk");
 
         assert_eq!(
             sk.to_string(),
@@ -716,6 +739,9 @@ mod test {
     }
 
     #[test]
+    // In fuzzing mode the Y coordinate is expected to match the X, so this
+    // test uses invalid public keys.
+    #[cfg(not(fuzzing))]
     fn test_pubkey_serialize() {
         struct DumbRng(u32);
         impl RngCore for DumbRng {
@@ -824,7 +850,7 @@ mod test {
         assert_eq!(set.len(), COUNT);
     }
 
-    #[test]
+    #[cfg_attr(not(fuzzing), test)]
     fn pubkey_combine() {
         let compressed1 = PublicKey::from_slice(
             &hex!("0241cc121c419921942add6db6482fb36243faf83317c866d2a28d8c6d7089f7ba"),
@@ -844,6 +870,47 @@ mod test {
         assert_eq!(sum1.unwrap(), exp_sum);
     }
 
+    #[cfg_attr(not(fuzzing), test)]
+    fn pubkey_combine_keys() {
+        let compressed1 = PublicKey::from_slice(
+            &hex!("0241cc121c419921942add6db6482fb36243faf83317c866d2a28d8c6d7089f7ba"),
+        ).unwrap();
+        let compressed2 = PublicKey::from_slice(
+            &hex!("02e6642fd69bd211f93f7f1f36ca51a26a5290eb2dd1b0d8279a87bb0d480c8443"),
+        ).unwrap();
+        let compressed3 = PublicKey::from_slice(
+            &hex!("03e74897d8644eb3e5b391ca2ab257aec2080f4d1a95cad57e454e47f021168eb0")
+        ).unwrap();
+        let exp_sum = PublicKey::from_slice(
+            &hex!("0252d73a47f66cf341e5651542f0348f452b7c793af62a6d8bff75ade703a451ad"),
+        ).unwrap();
+
+        let sum1 = PublicKey::combine_keys(&[&compressed1, &compressed2, &compressed3]);
+        assert!(sum1.is_ok());
+        let sum2 = PublicKey::combine_keys(&[&compressed1, &compressed2, &compressed3]);
+        assert!(sum2.is_ok());
+        assert_eq!(sum1, sum2);
+        assert_eq!(sum1.unwrap(), exp_sum);
+    }
+
+    #[test]
+    fn create_pubkey_combine() {
+        let s = Secp256k1::new();
+
+        let (mut sk1, pk1) = s.generate_keypair(&mut thread_rng());
+        let (sk2, pk2) = s.generate_keypair(&mut thread_rng());
+
+        let sum1 = pk1.combine(&pk2);
+        assert!(sum1.is_ok());
+        let sum2 = pk2.combine(&pk1);
+        assert!(sum2.is_ok());
+        assert_eq!(sum1, sum2);
+
+        assert!(sk1.add_assign(&sk2.as_ref()[..]).is_ok());
+        let sksum = PublicKey::from_secret_key(&s, &sk1);
+        assert_eq!(Ok(sksum), sum1);
+    }
+
     #[test]
     fn pubkey_equal() {
         let pk1 = PublicKey::from_slice(
@@ -891,9 +958,14 @@ mod test {
         ";
 
         let s = Secp256k1::new();
-
         let sk = SecretKey::from_slice(&SK_BYTES).unwrap();
+
+        // In fuzzing mode secret->public key derivation is different, so
+        // hard-code the epected result.
+        #[cfg(not(fuzzing))]
         let pk = PublicKey::from_secret_key(&s, &sk);
+        #[cfg(fuzzing)]
+        let pk = PublicKey::from_slice(&PK_BYTES).expect("pk");
 
         assert_tokens(&sk.compact(), &[Token::BorrowedBytes(&SK_BYTES[..])]);
         assert_tokens(&sk.compact(), &[Token::Bytes(&SK_BYTES)]);
