@@ -135,6 +135,7 @@ pub use secp256k1_sys as ffi;
 #[cfg(any(test, feature = "rand"))] use rand::Rng;
 #[cfg(any(test, feature = "std"))] extern crate core;
 #[cfg(all(test, target_arch = "wasm32"))] extern crate wasm_bindgen_test;
+#[cfg(feature = "alloc")] extern crate alloc;
 
 use core::{fmt, ptr, str};
 
@@ -158,7 +159,7 @@ use core::ops::Deref;
 use core::mem;
 use ffi::{CPtr, types::AlignedType};
 
-#[cfg(feature = "global-context")]
+#[cfg(feature = "global-context-less-secure")]
 pub use context::global::SECP256K1;
 
 #[cfg(feature = "bitcoin_hashes")]
@@ -1269,7 +1270,7 @@ mod tests {
 
     }
 
-    #[cfg(feature = "global-context")]
+    #[cfg(feature = "global-context-less-secure")]
     #[test]
     fn test_global_context() {
         use super::SECP256K1;
